/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * Klasse zum Test der Klasse Ganzzahl.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class GanzzahlTest {

	/** Regel zum Auswerten erwarteter Exceptions. */
	@Rule
	public ExpectedException thrown = ExpectedException.none();

	/**
	 * Test: Konstruktor mit Attributname.
	 */
	@SuppressWarnings("unused")
	@Test
	public void testGanzzahlString() {
		Assert.assertEquals((Long) 0L, new Ganzzahl("Heinz").get());
		thrown.expect(IllegalArgumentException.class);
		new Ganzzahl(null);
	}

	/**
	 * Test: Konstruktor mit ATtributname und Wert.
	 */
	@SuppressWarnings("unused")
	@Test
	public void testGanzzahlStringLong() {
		Assert.assertEquals((Long) 0L, new Ganzzahl("Heinz", 0L).get());
		Assert.assertEquals((Long) (-1L * 10L), new Ganzzahl("Heinz", -10L).get());
		Assert.assertEquals((Long) 10L, new Ganzzahl("Heinz", 10L).get());
		Assert.assertEquals((Long) Long.MAX_VALUE, new Ganzzahl("Heinz", Long.MAX_VALUE).get());
		Assert.assertEquals((Long) Long.MIN_VALUE, new Ganzzahl("Heinz", Long.MIN_VALUE).get());

		thrown.expect(IllegalArgumentException.class);
		new Ganzzahl(null, 27L);
	}

	/**
	 * Test: Attributname ermitteln.
	 */
	@Test
	public void testGetName() {
		final Ganzzahl zahl = new Ganzzahl("Heinz");
		Assert.assertEquals("Heinz", zahl.getName());
	}

	/**
	 * Test: Serialisierung und Deserialisierung.
	 */
	@Test
	public void testSerialization() {
		try {
			Ganzzahl zahl = new Ganzzahl("ABC", 12L);
			Assert.assertEquals("ABC", zahl.getName());

			final ByteArrayOutputStream byteOut = new ByteArrayOutputStream(1000);
			final ObjectOutputStream output = new ObjectOutputStream(byteOut);
			output.writeObject(zahl);
			final byte[] bytes = byteOut.toByteArray();

			final ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(bytes));
			zahl = (Ganzzahl) input.readObject();
			Assert.assertEquals((Long) 12L, zahl.get());
		} catch (final ClassNotFoundException | IOException e) {
			Assert.fail(e.getLocalizedMessage());
		}
	}

	/**
	 * Test: Wert setzen.
	 */
	@Test
	public void testSet() {
		final Ganzzahl zahl = new Ganzzahl("Heinz");
		zahl.set(10L);
		Assert.assertEquals((Long) 10L, zahl.get());

		thrown.expect(NullPointerException.class);
		zahl.set(null);
	}
}
